% Plots impulse responses generated from BVAR model
% 
% Required inputs:
%
%   varnames.mat:           
%   estimates_SR
%               
%
% Program will create graphs for all shocks in 'impresdrawstore', i.e. for
% all elements in the 3rd dimension.
%
% Last modified: 19.08.2019
%##########################################################################
tic; clear all; clc; close all;

%  add path of required Matlab functions
addpath('functions');

%  specify name of dataset
addpath('data');

%  specify name of estimation/forecast output directory
addpath('output');

% load variable names
load('varnames.mat');

%% specify settings
%

shocksize=[0.0025];         % for comparison of impulse responses use a fixed shock size
                            % use [] for unscaled IRFs


bands=[16 84];              % percentiles for credible intervals
shockvar=[9];               % row vector of VAR position(s) of variable(s) to be shocked; 
                            % the label of the shock from 'varnames';  
                            % if custom names are desired, the cell array 
                            % 'varnames' must be adjusted accordingly.
                            
%load('shockedvariables');   % Alternatively can be recovered from
                            % 'shockedvariables'
                            
shockselect=[9];             % row vector, selects shocks to be displayed 
                            % from the 3rd dimension of imprespdrawstore; 
                            % if empty --> choose all
                            
nhor=24;                    % horizon for impulse responses

modeplot=0;                 % switch indicating whether impulse responses 
                            % at posterior mode are to be graphed
                            
frypagan=0;                 % switch indicating wheter impulse responses for
                            % median model selected by Fry-Pagan procedure
                            % are to be graphed

% get transformation matrix
set.dataFile = [pwd '\data\BVAR_JMCB_noR10.csv'];

%  date of first observation in data set - format [YYYY Q]
set.dateFirstQ = [1991 1];

%  start date of forecasts - format [YYYY Q] or empty 
set.dateStartQ = [2000 2];

%  end date of forecasts
set.dateLastQ = [2014 3];

set.dataFreq = 'Q';

load('info.mat');
L=info_out(1);
BVAR_setup;

% load output from estimation procedure and retrieve some information
load('estimates_SR');
%% Generate impulse responses
irfmatstore=NaN(nhor+1,N,N,size(res.mcmc.beta,3));

for ii=1:size(res.mcmc.beta,3)
    
    beta=squeeze(res.mcmc.beta(:,:,ii));
    sigma=squeeze(res.mcmc.sigma(:,:,ii));
    B0draw=squeeze(res.mcmc.B0(:,:,ii));
    
    irfmat=NaN(nhor+1,N,N);

    for j=1:N
        irfmat(:,:,j)=bvarIrfSR(beta,sigma,j,nhor,B0draw,1,shocksize);
    end

    irfmatstore(:,:,:,ii)=irfmat;
        
end

irfmatstore_sav = irfmatstore;

% Scale variables in 4*log-levels back to log-levels
irfmatstore=rescale_series(irfmatstore,1,data.trafoMat,set.dataFreq);

%% Plots of median IRF with percentiles
%

irfmedstore=NaN(nhor+1,3*N,size(shockselect,2));

iFigure = 1;

for i=1:size(shockselect,2)

     irfmat=NaN(nhor+1,N,3);

     % pulling out the impulse responses variable by variable   
     for j=1:N
       
        temp=squeeze(irfmatstore(:,j,shockselect(1,i),:))*100;
        irfmat(:,j,:)=[prctile(temp,50,2) prctile(temp,bands,2)];
        for kk=1:3
            irfmedstore(:,3*(j-1)+kk,i)=irfmat(:,j,kk);
        end

     end

     shockname=varnames(shockvar(i));
     addstring='(median)';
     legstring='median';
          
     bvarIrfPlot(irfmat(1:nhor,:,:),iFigure,[],[],shockname,varnames,[]);

     iFigure=iFigure+1;

     % choose median model (here fit over fullh horizon)
     if frypagan==1
         irfmin=irf_frypagan_mod(ifrmatstore);

         temp=NaN(nhor,N,4);
         temp(:,:,2:4)=irfmat;
         for j=1:N           
            temp(:,j,1)=squeeze(irfmatstore(:,j,shockselect(1,i),irfmin))*100;
         end
         irfmat=temp; 

         addstring='(median model)';
         legstring={'median model','median'};
     end
     
     varselect=[1 3 5 7; 2 4 6 8; 10 13 16 19; 11 14 17 20; 12 15 18 21];
     
     for nirf=1:rows(varselect)

      bvarIrfPlotFill(irfmat(1:nhor,:,:),iFigure,varselect(nirf,:),[],shockname,varnames,[]);   
      iFigure=iFigure+1;

     end     
     
     varselect=[9 22 23];
     
     for nirf=1:rows(varselect)

      bvarIrfPlotFill(irfmat(1:nhor,:,:),iFigure,varselect(nirf,:),[],shockname,varnames,[]);           
      iFigure=iFigure+1;

     end     

end      

%%
imprespdrawstore = irfmatstore;
save([pwd '\output\imprespdrawstoreSR.mat'],'imprespdrawstore','-v7.3');