% Estimation of a Bayesian VAR with endogenous prior selection.
%
% Approach following Giannone, Lenza and Primiceri (2015), Prior
% Selection in Vector Autoregressions, Review of Economics and
% Statistics, 97(2), 436-451.
%
% Code by Martin Mandler and Michael Scharnagl based on code from Giorgio
% Primiceri at 
%
%   http://faculty.wcas.northwestern.edu/~gep575/research.html
%
% Last updated 14.08.2019
%
% #########################################################################
tic; clear; clc; close all;

%  add path of required Matlab functions
addpath('functions');

%  add path of data 
addpath('data');

%  specify name of output directory
addpath('output');

%  specify name of dataset
set.dataFile = [pwd '\data\BVAR_JMCB_noR10.csv'];

% specify name of output file for model specification
infoFile=[pwd '\output\info'];

% frequency of observations 'M', 'Q' or 'A'
set.dataFreq = 'Q';

%  date of first observation in data set - format [YYYY Q/M] or [YYYY 0]
%  for annual data
set.dateFirstQ = [1991 1];

%  start date of analysis - format [YYYY Q/M] or [YYYY 0] for annual data 
%  or empty (if empty it will be the first date for which observations for 
%  all variables are available)
set.dateStartQ = [2000 2];      % data starts in 1999Q1 and VAR has 5 lags

%  end of estimation period format [YYYY Q/M] or [YYYY 0] for annual data
set.dateLastQ = [2014 3];

%% Setting specification parameters 
%

load('randgen.mat');
rng(s);             % seed random number generator as used in the estimation in the paper

%s = rng;

L = 5;              % number of lags in the VAR

pos=[];             % position of variables already in first differences; for 
                    % these the prior mean will be set equal to zero if the
                    % Minnesota is chosen;
                    
ar1prior=0;         % ar1 prior = 1 overrides Minnesota, and makes "pos" meaningless
                    % and imposes coefficients from univariate AR(1) as prior means
                    % of first own lag in VAR
 
reps = 3000000;        % number of mcmc draws;   
burn = 2995000;        % discarded draws;   
                       % reps must be at least r3*[number of draws to be 
                       % retained - burn] see below;
                       % effective burn in for r3 is [reps - burn]
                       % effective burn in for saving without spacing is 
                       % equal to burn.

savstep = [150 250 500];    
%savstep = [];         % row vector of maximum length 3 [r1 r2 r3]; determines
                       % that each r1th r2th and r3th draw of VAR parameters
                       % from the final (reps-burn)*r draws is to be saved
                       % in addition to each of the final (reps-burn) draws;
                       % if savstep = [] the only the last [reps - burn]
                       % draws will be saved.
                
hyperpriors=1;      % If =1 hyperparameters are treated as random variables, 
                    % if =0 hyperparameters are treated as fixed and do not enter
                    % value of marginal likelihood.
                    
hypfix=0;           % If =1 fixes hyperparamters in mcmc at posterior mode 
                    % and runs only Gibbs sampler for VAR parameters.
                    
mcmc=1;             % mcmc=1 implies that MCMC alogorith is run; 
                    % if hypfix=0 and MCMCdraws=1 there will be draws for 
                    % both the VAR coefficients and the hyperparameters;
                    % if hypfix=1 only the VAR coefficients are drawn;
                    % if MCMCdraws=0 when only the hyperparamers are drawn. 
                    % If MCMCdraw=0 and hypfix=1 there will be no draws and 
                    % the setting will automatically revert to mcmc=0.
                    
MCMCdraw=1;        % MCMCdraw=1 draws the VAR parameters.                    
                    
step=1.0;          % step length for MCMC - choose to get an acceptance ratio of around between 25% and 30%

Vc=10e6;            % prior variance on constants

                    % NEXT:
                    % switches for hyperparameters to be estimated; =1 
                    % implies estimation =0 implies that hyperparameters
                    % are fixed according to default settings in
                    % BVAR_default_options
                    
MNpsi=1;            
MNalpha=0;
MNlambda=1;
sur=1;              % single unit root prior (initial-dummy-observations prior)
noc=1;              % sum of coefficients prior

suron=1;            % switches sur and noc priors on even if not optimized,
nocon=1;            % i.e. if sur=0 and noc=0. Hyperparameters will be set according
                    % to default settings. Choice is irrelevant if
                    % sur=1/noc=1.

                    
%% Consistency checking
%
if ~isempty(savstep)
   for j=1:size(savstep,2)
     if (reps-burn)*savstep(1,j)>reps
         disp('Save steps spaced too wide for targeted overall number of draws.');
         return;
     end
   end
end

%% Load and prepare data for analysis
%
BVAR_setup;

N = size(Y,2);      % number of variables

% save variable names for later use
varnames=data.nameSer;
save([pwd '\output\varnames.mat'],'varnames');

% save date vector and lag length
info_out=[L; dateVec];   
save(infoFile,'info_out');

%% load default options for VAR model
%
BVAR_default_options;

%% find mode of posterior/marginal likelihood for hyperparametersand run MCMC
%
res = GLP_estimation_adjborders(defaultpar,Y,X,L,pos,hyperpriors,mcmc,step,reps,burn,mn,Vc,sur,noc,suron,nocon,MCMCdraw,hypfix,savstep,ar1prior,1);


%% save estimation output
%
save([pwd '\output\estimates.mat'],'res');
save([pwd '\output\randgen.mat'],'s')
disp(['Elapsed time:   ', num2str(toc)]);

%% extract output for the 500th savesteps and save them 
% 
res_store = res;
clear res;
res.mcmc.beta = res_store.mcmc.beta4;
res.mcmc.sigma = res_store.mcmc.sigma4;

save([pwd '\output\estimates_500.mat'],'res');

%% call convergence diagnostics for MCMC
% 
if MCMCdraw==1
    pplot = 1; 
    [ GCD, s_nc, IEF ] = BVAR_ConvTest(res,N,L,pplot);
end

elapsedtime=toc; 
