% BVAR_setup.m
% #########################################################################
% Checks consistency of choices of options and start/end dates and sets up
% data matrices for BVAR.
% #########################################################################
%
% required inputs:
% set.dateFirstQ = [YYYY Q]     date of first observation in data set
%                  [YYYY M]  
% set.dateStartQ = [YYYY Q]     start date for estimation; must be at least
%                  [YYYY M]     dateFirstQ+Lags
% 
% set.dateLastQ  = [YYYY Q]     end date for estimation
%                  [YYYY M]
%
% set.dataFreq   = 'A' 'Q' 'M'  data frequency                
%
% L:    number of lags
%
% Modified version of code by Klemens Hauzenberger.
%
% Written by Martin Mandler and Michael Scharnagl, Deutsche Bundesbank
% 
% Loads data from comma seperated csv file

%% to do: check loading of quarterly and annual data

%  check for monthly, quarterly annual frequency
if strcmp(set.dataFreq,'M')
    dataFreq=2;
elseif strcmp(set.dataFreq,'A')
    dataFreq=0;
else
    dataFreq=1;
end

% import table
temptable = readtable(set.dataFile);

% series names
tempnames = temptable.Properties.VariableDescriptions;
tempnames = tempnames(2:end);

for kk = 1 : length(tempnames)
    cellContents = tempnames{kk};
    % Truncate and stick back into the cell
    tempnames{kk} = cellContents(26:end);
end

data.raw.nameSer = tempnames;

% transformation flags
data.raw.trafoMat = csvread(set.dataFile,4,1,[4,1,11,23]);

%  load raw data (hard-coded for first data entry in row 14) 
if dataFreq==2
    shift=12;
elseif dataFreq==0
    shift=1;    
else
    shift=4;   
end

data.raw.y = csvread(set.dataFile,13,1);

% read and construct date vector
data.raw.dateMat = table2array(temptable(13:end,1));

data.raw.dateMat = datenum(data.raw.dateMat,'dd.mm.yyyy');
data.raw.dateMat = datevec(data.raw.dateMat);
data.raw.dateMat = data.raw.dateMat(:,1:2);

if dataFreq==2
 data.dateVec = data.raw.dateMat(:,1)+(data.raw.dateMat(:,2)-1)./12;
elseif dataFreq==0
 data.dateVec = data.raw.dateMat(:,1)+data.raw.dateMat(:,2);
else
 data.dateVec = data.raw.dateMat(:,1)+(data.raw.dateMat(:,2)-3)./12;
end

%% PREPARE DATA FOR ANALYSIS
%
data.y = NaN(size(data.raw.y,1),sum(data.raw.trafoMat(6,:)));
data.nameSer = cell(1,sum(data.raw.trafoMat(6,:)));
data.trafoMat = zeros(size(data.raw.trafoMat,1), ...
    sum(data.raw.trafoMat(6,:))); 

for i = 1:size(data.raw.y,2)
     
    % check if series 'i' is selected
    if data.raw.trafoMat(6,i) == 1
        
        % name of series and transformation matrix - reordered given
        % information in excel sheet
        data.nameSer{data.raw.trafoMat(5,i)} = data.raw.nameSer{i};
        data.trafoMat(:,data.raw.trafoMat(5,i)) = data.raw.trafoMat(:,i);
        
        % log transformation (periods x log-levels)
        if data.raw.trafoMat(2,i) == 1 && data.raw.trafoMat(3,i) == 0
            data.y(:,data.raw.trafoMat(5,i)) = shift*log(data.raw.y(:,i));
        
        % log differences or annual growth rates   
        elseif data.raw.trafoMat(2,i) == 1 && data.raw.trafoMat(3,i) > 0
            if data.raw.trafoMat(3,i) == 1
                data.y(2:end,data.raw.trafoMat(5,i)) = ...
                    (log(data.raw.y(2:end,i))- ...
                    log(data.raw.y(1:end-1,i)));
            elseif data.raw.trafoMat(3,i) == 2
                data.y(shift+1:end,data.raw.trafoMat(5,i)) = ...
                    (log(data.raw.y(shift+1:end,i))- ...
                    log(data.raw.y(1:end-4,i)));%*100;
            end
            
        % annualized growth rate or annual growth rate of "raw" series
        elseif data.raw.trafoMat(2,i) == 0 && data.raw.trafoMat(3,i) > 0
            if data.raw.trafoMat(3,i) == 1
                data.y(2:end,data.raw.trafoMat(5,i)) = ...
                    (data.raw.y(2:end,i)./data.raw.y(1:end-1,i)-1)*shift; %*400;
            elseif data.raw.trafoMat(3,i) == 2
                 data.y(5:end,data.raw.trafoMat(5,i)) = ...
                    (data.raw.y(shift:end,i)./data.raw.y(1:end-4,i)-1);%*100;
            end
                        
        % no transformation
        else
            data.y(:,data.raw.trafoMat(5,i)) = data.raw.y(:,i);
        end
    end 
      
end 
 
N = size(data.y,2); % number of variables in y

Y = data.y;
dateVec = data.dateVec;

if ~isempty(set.dateStartQ)
    temp.idxT = ((set.dateStartQ(1)+(set.dateStartQ(2)-1)/shift)- ...
        dateVec(1))*shift+1-L;
    if temp.idxT<1
     disp('Estimation start not compatible with chosen lag order!');
     return
    end
    if temp.idxT > 1
        Y = Y(temp.idxT:end,:); 
        dateVec = dateVec(temp.idxT:end,:);
    end
end



%% build data matrices
X = []; 
 
for i = 1 : L 
    X = [ X lag0(Y,i) ]; 
end 

X = [ X ones(size(Y,1),1) ];
Y = Y(L+1:end,:);
X = X(L+1:end,:);
