% Identification of shocks through sign and zero restrictions in a Bayesian VAR
% model.
% 
% Required inputs:
% Files saved by BVAR_main
%
% \output\estimates.mat:    beta and sigma matrices from the MCMC algorithm
%
% \output\info.mat:         number of lags in VAR
%
% Saves structure res.mcmc. ... containing VAR coefficients (beta),
% covariances (sigma) and rotation matrices (A0) only for those draws for
% which sign restrictions were satisfied. Variables are formatted and
% structures as in BVAR_revised.m
%
% Written by Martin Mandler, Deutsche Bundesbank.
%
% Last modified: 19.08.2019
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic; clear; clc; close all;

%  add path of required Matlab functions
addpath('functions');

%  specify name of dataset
addpath('data');

%  specify name of estimation/forecast output directory
addpath('output');

%  specify name of dataset
set.dataFile = [pwd '\data\BVAR_JMCB_noR10.csv'];

% data frequency
set.dataFreq = 'Q';

%  date of first observation in data set - format [YYYY Q]
set.dateFirstQ = [1991 1];

%  first date for which observations for all variables are available)
set.dateStartQ = [2000 2];

%  end date of forecasts
set.dateLastQ = [2014 3];

%%
% load output from estimation procedure and retrieve some information
load('estimates_500.mat');

betadraw=res.mcmc.beta;
sigmadraw=res.mcmc.sigma;

clear res;

% load additional information from estimation
load('info.mat');
L=info_out(1);

% build data matrices for VAR (X matrix is needed to build state-space
% system
BVAR_setup;

nhorizon=24;

shockvar=[9];
shocksize=[0.0025];   %[0.0025] sets size of shock to 25 bp

maxdraws = 10000;     % max. number of attempts to find a rotation matrix for a reduced form VAR draw
                     % adjust to obtain about the desired number of
                     % accepted draws

%% Zero and sign restrictions
% set zero restrictions (horizon, variable, shock)
% 1 is impact period, 2 is first period
% 1 indicates a binding restriction, zero a non-binding 
[k, N, ndraws] = size(betadraw);

zeror_mat=zeros(1,N,N);

% set sign restrictions (horizon, variable, shock)
% 1 is impact period, 2 is first period
% +1/-1 indicates restrictions, 0 leaves resonse free
signr_mat=zeros(1,N,N);

% monetary policy shock
signr_mat(1,[1:8],9)=-1;

% important: positive SR of own variable must be imposed on all shocks to be identified
% identifies monetary policy shock as a positive innovation in policy rate
signr_mat(1,9,9)=1;

% restrictions on A0 concerning monetary plicy shock
% [effect from variable, effect to variable (shock)]

% zero restrictions
Azeror_mat = zeros(N,N);

% sign restrictions
Asignr_mat = zeros(N,N);

% identify monetary policy reaction function: positive response to Y, P
% note that signs are reversed because Y, P are on LHS
Asignr_mat(1:8,9)=-1;
Asignr_mat(9,9)=1;


%%
% transforms restrictions matrices into format used by Arias et al. (2014)
[Zres,Sres,ix]=GenRes_ARW_setup(zeror_mat,signr_mat,Azeror_mat,Asignr_mat,N);

%% Sign restrictions

[A0drawstore,B0drawstore,betadrawstore,sigmadrawstore,model_acc,SRsuccess]=GenRes_ARW_MP(betadraw,sigmadraw,Zres,Sres,ix,maxdraws,1,[]);
% note B0drawstore is the inverse of A0 multiplied by diagonal matrix of covariances and transposed, ie. (A0drawstore)^-1*D*(A0drawstore)^-1' =
% Sigma with D as a diagonal matrix; impact effects of shocks on y (as a
% column vector) are B0'

%% Generate impulse responses

irfmatstore=NaN(nhorizon+1,N,N,size(betadrawstore,3));

for ii=1:size(betadrawstore,3)
    
    beta=squeeze(betadrawstore(:,:,ii));
    sigma=squeeze(sigmadrawstore(:,:,ii));
    B0draw=squeeze(B0drawstore(:,:,ii));
    
    irfmat=NaN(nhorizon+1,N,N);

    for j=1:N
       irfmat(:,:,j)=bvarIrfSR(beta,sigma,j,nhorizon,B0draw,1,shocksize);
    end

    irfmatstore(:,:,:,ii)=irfmat;
        
end
toc;
%% make plots
% 

Nshocks=size(shockvar,2);
bands=[16 64];

irfmatnew=irfmatstore;
  
% Scale variables in 4*log-levels back to log-levels
imprespmodestore=rescale_series(irfmatnew,1,data.trafoMat,set.dataFreq);
imprespdrawstore=rescale_series(irfmatnew,1,data.trafoMat,set.dataFreq);
    
% plot IRFs 
% Determine how many columns and rows are needed to display the IRF

Ncols=fix(sqrt(N));

if Ncols^2<N
    Nrows=Ncols+1; 
    if Ncols*Nrows<N
        Ncols=Ncols+1;
    end
else
    Nrows=Ncols;  
end
 
iFigure=1;
TT=1:nhorizon+1;
    
% plots of median IRF with percentiles
irfmedstore=NaN(nhorizon+1,N,3,Nshocks);
varnames=data.nameSer;

for i=1:Nshocks
 
    irfmat=NaN(nhorizon+1,N,3);
    % pulling out the impulse responses variable by variable   
    for j=1:N

     temp=squeeze(imprespdrawstore(:,j,shockvar(i),:));
     irfmat(:,j,:)=[prctile(temp,50,2) prctile(temp,bands,2)];
     irfmedstore(:,j,:,i)=squeeze(irfmat(:,j,:));


    end
     
    shockname=data.nameSer(shockvar(i));
    addstring='(median)';
    legstring='median';
    bvarIrfPlot(irfmat,iFigure,[],addstring,shockname,data.nameSer,legstring);

    iFigure=iFigure+1;
           
end       

%% save results 
%
acc_rate=sum(model_acc)/((size(betadraw,3)*maxdraws));
disp(['Proportion of accepted draws: ' num2str(acc_rate)]);


res.mcmc.beta=betadrawstore;
res.mcmc.sigma=sigmadrawstore;
res.mcmc.B0=B0drawstore;
res.mcmc.A0=A0drawstore;

save([pwd '\output\imprespdrawstoreSR.mat'],'imprespdrawstore','-v7.3');
save([pwd '\output\estimates_SR.mat'],'res','-v7.3');