function res = GLP_estimation_adjborders(defaultpar,Y,X,L,pos,hyperpriors,mcmc,step,reps,burn,mn,Vc,sur,noc,suron,nocon,MCMCdraw,hypfix,...
    savstep,ar1prior,nchains)  
% GLP_estimation: Estimation of BVAR with endogenous selection of
% hyperparameters
%
% Estimation of BVAR with optimization over hyperparameters 
% Follows Giannone, Lenza and Primiceri (2014), Prior Selection for Vector
% Autoregressions, Review of Economics and Statistics (forthcoming).
%
% Based on Matlab code by Giannone, Lenza and Primiceri
% http://faculty.wcas.northwestern.edu/%7Egep575/GLPreplicationWeb.zip
%
% Modified by Martin Mandler and Michael Scharnagl, Deutsche Bundesbank.
%
% Last modified: 19.08.2019
%
% INPUTS:
%
% Y: matrix of dependent variables  [T x N]
%
% X: matrix of explanatory variables [T x (N x L) + 1]
%
% L: lags
%
% pos: position of variables in first differences; prior mean on
% coefficient on first lag will be set to zero instead of one under
% Minnesota prior. Default [].
%
% defaultpar:   structure containing the default values for the
%               hyperparameters alpha, lambda, miu and theta
%
% hyperpriors: 0 = no priors on hyperparameters; hyperparameters are treated 
%                  as given in optimization over them.
%              1 = reference priors on hyperparameters; hyperparameters are
%                  treated as random variables. Default [1].
%
% mcmc: 0 = no MCMC algorithm. 
%       1 = run MCMC algorithm after optimization. Default [1].
%
% hypfix: 0 = hyperparamters included in MCMC. Default [0] 
%         1 = hyperparameters fixed in MCMC at posterior mode
%
% step: scaling variable in MCMC algorithm (set to achieve approx. 25%
%       acceptance rate). Default [1.0]
%
% reps: number of draws in MCMC algorithm [20000]
%
% burn: number of burn-in draws [reps/2]
%
% mn.psi: 0 = ML not maximized with respect to Psi; Psi set equal to
%             variance of AR(1) for each variable
%         1 = ML maximized with respect to psi. Default [1].
%
%         Psi is scale parameter on IW prior on covariance matrix of VAR
%         residuals.
%
% mn.alpha 0 = ML not maximized with respect to alpha; alpha = 2; Default [0].
%          1 = ML maximized with respect to alpha
%
%         Alpha is lag decay parameter in Minnesota prior.
%
% mn.lambda 0 = ML not maximized with respect to lambda; lambda = 0.2 if no
%           1 = ML maximized with respect to lambda. Default [1].
%
% Vc: prior variance of constants; Default [10e6].
%
% sur: 1 = ML maximized with respect to theta. Default [1].
%      0 = Single unit root prior is off.
%      
% noc: 1 = ML maximized with respect to miu. Default [1].
%      0 = No cointegration prior is off.
%
%
% MCMCdraw:  0 = no draws for beta and sigma during MCMC steps. 
%            1 = draws for each MCMC step
%            Default is MCMCdraw=1 if mcmc=1.
%
% savstep: row vector stating which x-th draw of the final draws should be
%          saved (see main program). Maximum size is [x1 x2 x3].
%
% ar1prior:     Minnesota-type prior using coefficients from estimatede AR1
%               process as prior for first own lag of variables.
%
% nchains:      number of MCMC chains. Default is [1]
%
%
% ########################################################################

%% Check whether savesteps is in effect.
if exist('savstep','var') && ~isempty(savstep)
   sstep=1;
   nsteps=size(savstep,2);
   if nsteps>3
    disp('Savstep must not have more than three elements.');
    return
   end   
   
   for j=1:nsteps
    Ndd(j)=reps-(reps-burn)*savstep(j);
   end
   
else
    sstep=0;
end

if isempty(nchains)
    nchains=1;
end

%% Some parameters
N=size(Y,2);
T=rows(X);
k=size(X,2);

Y0=mean(Y(1:L,:));

% set parameters of hyperpriors

if hyperpriors==1
    
    mode.lambda=.2;     % hyperpriors modes
    mode.miu=1;
    mode.theta=1;
        
    sd.lambda=.4;       % hyperpriors std
    sd.miu=1;
    sd.theta=1;
        
    scalePSI=0.02^2;    % scale and shape of the IG on psi/(d-n-1)    
    
    priorcoef.lambda=GammaCoef(mode.lambda,sd.lambda,0);  % coefficients of hyperpriors
    priorcoef.miu=GammaCoef(mode.miu,sd.miu,0);
    priorcoef.theta=GammaCoef(mode.theta,sd.theta,0);    
    priorcoef.alpha.PSI=scalePSI;
    priorcoef.beta.PSI=scalePSI;
    
else
    priorcoef=[];
end

% bounds for maximization
MIN.lambda = 0.1;
MIN.alpha = 0.1;
MIN.theta = 0.1;
MIN.miu = 0.1;
MAX.lambda=5;
MAX.miu=50;
MAX.theta=50;
MAX.alpha=5;


%% Minnesota Prior mean - note that constants are ordered last
b=zeros(k,N);
diagb=ones(N,1);
diagb(pos)=0;

% starting values for minimization
lambda0=.2;     % std of MN prior
theta0=1;       % std of sur prior
miu0=1;         % std of noc prior
alpha0=2;       % lag-decaying parameter of the MN prior

% If you wish to synch them with the default settings use the following:
%alpha0=defaultpar.alpha;
%lambda0=defaultpar.lambda;
%miu0=defaultpar.miu; 
%theta0=defaultpar.theta;

% residual variance of AR(1) for each variable and AR(1) prior if chosen
SS=zeros(N,1);
for i=1:N
    ar1=ols1(Y(2:end,i),[Y(1:end-1,i),ones(T-1,1)]);
    SS(i)=ar1.sig2hatols;
    % AR(1) priors
    if ar1prior==1
       diagb(i)=ar1.bhatols(1,1); 
    end    
end

b(1:N,:)=diag(diagb);

MIN.psi=SS./100;
MAX.psi=SS.*100;
psi0=SS;

if mn.psi==1
    inpsi=-log((MAX.psi-psi0)./(psi0-MIN.psi));
elseif mn.psi==0
    inpsi=[];
end

if mn.alpha==1
    inalpha=-log((MAX.alpha-alpha0)/(alpha0-MIN.alpha));
elseif mn.alpha==0
    inalpha=[];
end

if mn.lambda==1
   inlambda=-log((MAX.lambda-lambda0)/(lambda0-MIN.lambda)) ;
elseif mn.lambda==0
    inlambda=[];
end

if sur==1
    intheta=-log((MAX.theta-theta0)/(theta0-MIN.theta));
elseif sur==0
    intheta=[];
end

if noc==1
    inmiu=-log((MAX.miu-miu0)/(miu0-MIN.miu));
elseif noc==0
    inmiu=[];
end

x0=[inlambda;inpsi;intheta;inmiu;inalpha];

% If no optimization over any hyperparameters set a switch to jump over
% parts of the code.
if isempty(x0)
    noopt=1;
    if noopt==1 && MCMCdraw==0
        mcmc=0;
    end
else 
    noopt=0;
end

H0=10*eye(length(x0));          % initial guess for the inverse Hessian

%% Maximization of the posterior of the hyperparameters

if noopt==0
    [fh,xh,gh,H,itct,fcount,retcodeh] = csminwel('logMLVAR_forminVo1',x0,H0,...
        [],1e-16,1000,defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,hyperpriors,priorcoef);

    res.itct=itct;   

    % Retrieve VAR coefficients, covariance matrix and residuals at posterior
    % mode.
    [res.logML,res.betahat,res.sigmahat,res.epshat,res.d,res.omega] = ...
        logMLVAR_forminVo1(xh,defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,hyperpriors,priorcoef);

    res.SSar1=SS;
    
    % Set theta and miu to arbitrary values if they were not included
    % in the optimization and not given as fixed hyperparameters. These
    % values will not matter later.
    res.theta=MAX.theta;
    res.miu=MAX.miu;

    % If hyperparameters were estimated, use the results from the
    % estimation, else use default values.
    if mn.lambda==0
        res.lambda=defaultpar.lambda;
        jshift=0;                       % indicates whether lambda is included in maximization
    elseif mn.lambda==1
        % Overall tightness parameter of MN prior at the peak
        res.lambda=MIN.lambda+(MAX.lambda-MIN.lambda)/(1+exp(-xh(1)));
        jshift=1;
    end

    if mn.psi==1
        % diagonal elements of the scale matrix of the IW prior on the residual variance
        res.psi=MIN.psi+(MAX.psi-MIN.psi)./(1+exp(-xh(jshift+1:N+jshift)));
        if sur==1
            % sur prior at the peak
            res.theta=MIN.theta+(MAX.theta-MIN.theta)/(1+exp(-xh(N+jshift+1)));
            if noc==1
                % noc prior at the peak
                res.miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-xh(N+jshift+2)));  
            elseif noc==0 && nocon==1
                res.miu=defaultpar.miu;
            end
        elseif sur==0
            if suron==1
                res.theta=defaultpar.theta;
            end
            if noc==1
                % sur prior at the peak
                res.miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-xh(N+jshift+1)));
            elseif noc==0 && nocon==1
                res.miu=defaultpar.miu;
            end
        end
    elseif mn.psi==0
        res.psi=SS;
        if sur==1
            % sur prior at the peak
            res.theta=MIN.theta+(MAX.theta-MIN.theta)/(1+exp(-xh(jshift+1)));
            if noc==1;
                % noc prior at the peak
                res.miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-xh(jshift+2)));
            end
        elseif sur==0
            if noc==1
                % noc prior at the peak
                res.miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-xh(jshift+1)));
            end
        end
    end

    if mn.alpha==0
        res.alpha=defaultpar.alpha;
    elseif mn.alpha==1
        % lag-decay parameter of the MN prior at the peak
        res.alpha=MIN.alpha+(MAX.alpha-MIN.alpha)/(1+exp(-xh(end)));
    end

else 
   
    % No estimation of hyperparameters - this gives the posterior mode of
    % beta and sigma only.
    
    [res.logML,res.betahat,res.sigmahat,res.epshat,res.d,res.omega] = logMLVAR_formin_fixedprior(defaultpar,Y,X,L,T,N,b,SS,Vc,pos,suron,nocon,Y0);

end

%% run MCMC algorithm
%

if mcmc==1
    
    if noopt==0
        
        if hypfix==0    % hyperparameters are drawn alongside with VAR parameters
                        % setup for including the hyperparameters in the
                        % MCMC
            
            JJ=exp(xh)./(1+exp(xh)).^2;

            if mn.lambda==1
                JJ(1)=(MAX.lambda-MIN.lambda)*JJ(1);       
            end

            if mn.psi==1
                JJ(jshift+1:N+jshift)=(MAX.psi-MIN.psi).*JJ(jshift+1:N+jshift);
                if sur==1
                    JJ(N+jshift+1)=(MAX.theta-MIN.theta)*JJ(N+jshift+1);
                    if noc==1
                        JJ(N+jshift+2)=(MAX.miu-MIN.miu)*JJ(N+jshift+2);
                    end
                elseif sur==0
                    if noc==1
                        JJ(N+jshift+1)=(MAX.miu-MIN.miu)*JJ(N+jshift+1);
                    end
                end
            elseif mn.psi==0
                if sur==1
                    JJ(jshift+1)=(MAX.theta-MIN.theta)*JJ(jshift+1);
                    if noc==1
                        JJ(jshift+2)=(MAX.miu-MIN.miu)*JJ(jshift+2);
                    end
                elseif sur==0
                    if noc==1
                        JJ(jshift+1)=(MAX.miu-MIN.miu)*JJ(jshift+1);
                    end
                end
            end

            if mn.alpha==1
                JJ(end)=(MAX.alpha-MIN.alpha)*JJ(end);
            end


            JJ=diag(JJ);
            HH=JJ*H*JJ';

            % regularizing the Hessian (making sure it is positive definite)
            [V,E]=eig(HH);
            HH=V*abs(E)*V';
            
        end

        % recovering the posterior mode
        if mn.psi==1
            modepsi=res.psi;
        elseif mn.psi==0
            modepsi=[];
        end

        if mn.alpha==1
            modealpha=res.alpha;
        elseif mn.alpha==0
            modealpha=[];
        end

        if sur==1
            modetheta=res.theta;
        elseif sur==0
            modetheta=[];
        end

        if noc==1
            modemiu=res.miu;
        elseif noc==0
            modemiu=[];
        end

        if mn.lambda==1
            modelambda=res.lambda;
        elseif mn.lambda==0
            modelambda=[];
        end

        postmode=[modelambda;modepsi;modetheta;modemiu;modealpha];

        % initialize MCMC algorithm
        
        P=zeros(reps,length(xh),nchains);
        
        for cc=1:nchains
            
            % do not draw hyperparamters if they are fixed at posterior mode 
            if hypfix==1
                P(1,:,cc)=postmode;
                [logMLold,betadrawold,sigmadrawold]=logMLVAR_formcmcVo1(P(1,:,cc)',defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,...
                        MCMCdraw,hyperpriors,priorcoef);
            else
                logMLold=-10e15;
                while logMLold==-10e15  % this ensures that the algorithm does not start at the mode
                    
                    % overdispersed initialization based on a multivariate
                    % t- distribution as approximation to the posterior
                    
                    % scale matrix is HH^-1, degrees of freedom = 4 as in
                    % Gelman et al. (2014), p 319.
                                       
                                       
                    %P(1,:,cc) = st_rndmvt(postmode',HH,4,1);  %%% TO DO,
                    %reenable this line when multiple steps are used
                                        
                    P(1,:,cc)=mvnrnd(postmode,HH*step^2,1);                                           
                    
                    [logMLold,betadrawold,sigmadrawold]=logMLVAR_formcmcVo1(P(1,:,cc)',defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,...
                        MCMCdraw,hyperpriors,priorcoef);
                  
                end
            end
        end 
        
        % matrix to store the draws of the VAR coefficients if MCMCdraw
        if MCMCdraw==1
            res.mcmc.beta  = zeros(k,N,reps-burn,nchains);
            res.mcmc.sigma = zeros(N,N,reps-burn,nchains);

            % Determine size of storage matrices for draws saved by savstep
            if sstep==1
                if nsteps>=1
                    res.mcmc.beta2  = zeros(k,N,reps-burn,nchains);
                    res.mcmc.sigma2 = zeros(N,N,reps-burn,nchains);
                    if nsteps>=2            
                        res.mcmc.beta3  = zeros(k,N,reps-burn,nchains);
                        res.mcmc.sigma3 = zeros(N,N,reps-burn,nchains);
                            if nsteps==3
                                res.mcmc.beta4  = zeros(k,N,reps-burn,nchains);
                                res.mcmc.sigma4 = zeros(N,N,reps-burn,nchains);
                            end
                    end
                end
            end
        end

        % Metropolis iterations
        count=zeros(1,nchains);
        for i=2:reps
            for cc=1:nchains

                if i==100*floor(.01*i) && cc==nchains
                    if hypfix==0
                        accrate=count./i; 
                        disp(['Now running the ',num2str(i),'th mcmc iteration (out of ',num2str(reps),') AccRate: ',num2str(accrate)])
                    else 
                        disp(['Now running the ',num2str(i),'th mcmc iteration (out of ',num2str(reps),')'])
                    end
                end

                if hypfix==1    % do not draw hyperparamters, run Gibbs sampler for VAR parameteres

                    P(i,:,cc)=P(i-1,:,cc);

                    [junk,betadrawnew,sigmadrawnew]=logMLVAR_formcmcVo1(P(i,:,cc)',defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,...
                        MCMCdraw,hyperpriors,priorcoef);


                else            % draw hyperparameters and run Gibbs sampler for VAR coefficients afterwards

                    % draw candidate value
                    P(i,:,cc)=mvnrnd(P(i-1,:,cc),HH*step^2,1);

                    [logMLnew,betadrawnew,sigmadrawnew]=logMLVAR_formcmcVo1(P(i,:,cc)',defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,...
                        MCMCdraw,hyperpriors,priorcoef);

                    if logMLnew>logMLold 
                        logMLold=logMLnew;
                        count(1,cc)=count(1,cc)+1;
                    else
                        if rand(1)<exp(logMLnew-logMLold);
                            logMLold=logMLnew;
                            count(1,cc)=count(1,cc)+1;
                        else
                            P(i,:,cc)=P(i-1,:,cc);


                            if MCMCdraw==1 

                                [junk,betadrawnew,sigmadrawnew]=logMLVAR_formcmcVo1(P(i,:,cc)',defaultpar,Y,X,L,T,N,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,Y0,...
                                    MCMCdraw,hyperpriors,priorcoef);

                            end
                        end
                    end

                end

                % stores draws of VAR coefficients
                if MCMCdraw==1

                    if i>burn      
                        res.mcmc.beta(:,:,i-burn,cc) = betadrawnew;  
                        res.mcmc.sigma(:,:,i-burn,cc) = sigmadrawnew;   
                    end

                    if sstep==1

                        if i>Ndd(1) && mod(i-Ndd(1),savstep(1))==0
                            res.mcmc.beta2(:,:,fix((i-Ndd(1))/savstep(1)),cc) = betadrawnew;  
                            res.mcmc.sigma2(:,:,fix((i-Ndd(1))/savstep(1)),cc) = sigmadrawnew;   
                        end

                        if nsteps>=2 && i>Ndd(2) && mod(i-Ndd(2),savstep(2))==0 
                            res.mcmc.beta3(:,:,fix((i-Ndd(2))/savstep(2)),cc) = betadrawnew;  
                            res.mcmc.sigma3(:,:,fix((i-Ndd(2))/savstep(2)),cc) = sigmadrawnew;   
                        end

                        if nsteps==3 && i>Ndd(3) && mod(i-Ndd(3),savstep(3))==0 
                            res.mcmc.beta4(:,:,fix((i-Ndd(3))/savstep(3)),cc) = betadrawnew;  
                            res.mcmc.sigma4(:,:,fix((i-Ndd(3))/savstep(3)),cc) = sigmadrawnew;   
                        end

                    end
                end 
            end
        end

        % store the draws of the hyperparameters
        if sstep==1
            res.mcmc.savHyp=P(min(Ndd)+1:end,:,:);
        end

        if mn.lambda==1
            % MN prior shrinkage parameter
            res.mcmc.lambda=P(burn+1:end,1,:);   
        end

        if mn.psi==1
            % diagonal elements of the scale matrix of the IW prior on the residual variance
            res.mcmc.PSI=P(burn+1:end,[jshift+1:N+jshift],:);
            if sur==1
                % sur prior
                res.mcmc.theta=P(burn+1:end,N+jshift+1,:);
                if noc==1
                    % noc prior
                    res.mcmc.miu=P(burn+1:end,N+jshift+2,:);
                end
            elseif sur==0
                if noc==1
                    % noc prior
                    res.mcmc.miu=P(burn+1:end,N+jshift+1,:);
                end
            end
        elseif mn.psi==0
            if sur==1
                % sur prior
                res.mcmc.theta=P(burn+1:end,jshift+1,:);
                if noc==1
                    % noc prior
                    res.mcmc.miu=P(burn+1:end,jshift+2,:);
                end
            elseif sur==0
                if noc==1
                    % noc prior
                    res.mcmc.miu=P(burn+1:end,jshift+1,:);
                end
            end
        end

        if mn.alpha==1
            % lag decay parameter of the MN prior
            res.mcmc.alpha=P(burn+1:end,end,:);
        end

        % acceptance rate
        if hypfix==0
            res.mcmc.ACCrate=[count./reps sum(count,2)/(reps*nchains)]; 
        end
           
        % if hyperparameters are fixed at exogenously given value,
        % run only the Gibbs sampler for beta and sigma
    else  
        
        % matrix to store the draws of the VAR coefficients 
        res.mcmc.beta  = zeros(k,N,reps-burn,nchains);
        res.mcmc.sigma = zeros(N,N,reps-burn,nchains);

        % Gibbs sampler
        count=0;
        for i=2:reps

            for cc=1:nchains
                if i==100*floor(.01*i) && cc==nchains;
                            disp(['Now running the ',num2str(i),'th mcmc iteration (out of ',num2str(reps),')'])
                end

                [logMLnew,betadrawnew,sigmadrawnew]=logMLVAR_formcmc_fixedprior(defaultpar,Y,X,L,T,N,b,SS,Vc,pos,suron,nocon,Y0,MCMCdraw);

                count=count+1;

                % stores draws of VAR parameters
                if i>burn      
                    res.mcmc.beta(:,:,i-burn,cc) = betadrawnew;  
                    res.mcmc.sigma(:,:,i-burn,cc) = sigmadrawnew;   
                end            
            end
        end

    end 
        
        
   
      
end

end

