function [A0drawstore,B0drawstore,betadrawstore,sigmadrawstore,model_acc,SRsuccess] = GenRes_ARW_MP(betain,sigmain,Z,S,ix,maxdraws,const,infty)
% SignRes_ARW Identification by sign and zero restrictions
%
% Generates one draw of a rotating matrix satisfying given sign and zero
% restrictions following Arias, Rubio-Ramirez and Waggoner (2014), Inference
% based on SVARs identified with sign and zero restrictions, Dynare Working
% Paper no. 30.
%
% VAR notation: Y = X*beta + e with Y [T x nvars], X [T x k], 
%                   beta [k x nvars], e [T x nvars] reduced form residuals
%
% inputs:
%
% beta:     [k x nvars] matrix of VAR coefficients
%
% sigma:    [nvars x nvars] covariance matrix of VAR
%
% Z:    [nvars, nvars*nhorizon, nvars] [restrictions, unitvector, to shock]
%       empty restrictions (zero rows) will be deleted
%
% S:    [nvars, nvars*nhorizon, nvars] [restrictions, unitvector, to shock]
%
% both sets of restrictions are constructed from SignRes_ARW_setup.m
%
% ix is a vector containing the numbers of the VAR equations/shocks ordered 
% (i)% according to the number of zero restrictions per shock, (ii) ordered
% according to the number of sign restrictions per shock. The latter one is
% done in order to speed up the algorithm by checking the sing restrictions
% that are most difficult to satifsfy first, see p 17.
%
% const=1 indicates that a constant is included in the VAR (last row of
% beta)
%
% infty='infty' indicates zero or sign restrictions at infinite horizon
%               the code takes the final horizon i.e. bottom blocks of Z 
%               and S (same dimensions). 
%
% outputs:
%
% A0out:        rotation matrices for shocks which conform to sign and zero
%               restrictions
%
% flag_break:   indicator equalt to one when the sign restrictions were 
%               fulfilled and zero when they were not.
% 
% Written by Martin Mandler, Deutsche Bundesbank.
%
% Last modified: 05.05.2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[k,nvars,ndraws]=size(betain);

if ~exist('maxdraws','var')
    maxdraws=100;
elseif isempty(maxdraws)
    maxdraws=100;
end

if const==1
    nlags=(k-1)/nvars;
else
    nlags=k/nvars;
end

% horizon of restrictions
% correct maximum horizon (account for nvars restrictions on A0)
nhz=size(Z,1)/nvars-1;
nhs=size(S,1)/nvars-1;
nhmax=max([nhz,nhs]);

ns=NaN(nvars,1);     % ns(j): # sign restrictions for shock j
for i=1:nvars
    ns(i)=sum(sum(S(:,:,i)~=0));
end

A0drawstore=NaN(nvars,nvars,ndraws);
B0drawstore=NaN(nvars,nvars,ndraws);
betadrawstore=NaN(k,nvars,ndraws);
sigmadrawstore=NaN(nvars,nvars,ndraws);
model_acc=NaN(1,ndraws);

SRsuccess=0;

for ii=1:size(betain,3)

    if mod(ii,100)==0
        disp(['Draw: ' num2str(ii) ' Successes:' num2str(SRsuccess)]);
    end

    % pick reduced form VAR
    beta=squeeze(betain(:,:,ii));
    sigma=squeeze(sigmain(:,:,ii));
    
    % stack A0 from Choleski decomposition of covariance matrix in fron
    irfchol(1:nvars,1:nvars)=chol(sigma)\eye(nvars);
   
   
   
    % build impulse response matrix f(A_0,A_+) with different horizons stacked 
    % in blocks below each other         
    for i=1:nvars
        temp=bvarIrfsVo1(beta,sigma,i,nhmax-1,const);        
        for j=1:nhmax
            irfchol(nvars+nvars*(j-1)+1:nvars*(j+1),i)=temp(j,:)'; 
        end
    end
    
    % if infinite horizon is required, replace last rows of IRF matrix
    if strcmp(infty,'infty')

         %transform VAR into companion matrix form
         Gmat=zeros(nvars*nlags,nvars*nlags);
         Gmat(nvars+1:nvars*nlags,1:nvars*(nlags-1))=eye(nvars*(nlags-1));
         if const==1
            Gmat(1:nvars,1:k-1)=beta(1:end-1,:)';
         else
            Gmat(1:nvars,1:k)=beta'; 
         end

         % impulse responses at infinite horizon
         longRun=(eye(nvars*nlags)-Gmat)\eye(nvars*nlags);
         longRun=longRun(1:nvars,1:nvars)*chol(sigma)';

         % plug into matrix of impulse responses
         irfchol(end-nvars+1:end,:)=longRun;      

    end
    
    Msuccess=0;
    for kk=1:maxdraws
    
        % start with iteration to construct Q --> Algorithm 4, p. 17
        Qmat=[];    % starting value is empty matrix, see p. 12

        flag_break=0;
        j=1;
        while flag_break==0 && j<=nvars

            % cut rows of Zj containing only zeros
            Zj=squeeze(Z(:,:,ix(j)));
            Zj=Zj((sum(Zj,2)==1),:); 

            Nmat=null([Zj*irfchol; Qmat']);     % Theorem 4, step 2
            x=randn(nvars,1);                   % Theorem 4, sstep 3
            qj=Nmat*(Nmat'*x/norm(Nmat'*x));    % Theorem 4, step 4

            %cut rows of Sj containing only zeros
            Sj=squeeze(S(:,:,ix(j)));
            Sj=Sj(sum(Sj~=0,2)>0,:);

            %check whether current draw of qj satisfies sign restrictions, Section
            %3.7, p. 17

            if ns(ix(j))~=0         % no sign restrictions on current shock --> skip

                % multiply impulse response matrix by sign restrictions matrix; if
                % sign restrictions are satisfied, result should be > 0
                temp_irf=Sj*irfchol*qj;        
                a=(temp_irf>0);        
                
                if (max(a)==0)  % SR are not fulfilled for any variable for this shock
                    am=(temp_irf<=0);

                    if (min(am)==0) % SR are not fulfilled with reversed sign, new draw for beta and sigma required
                        flag_break=1;
                        
                    else % SR are fulfilled with reversed sign, switch sign on qj
                        qj=-qj;

                    end

                elseif (min(a)==0)  % SR are not fulfilled for some variables for this shock, new draw required
                    flag_break=1;
                end
            end
            
            Qmat=[Qmat qj];
            j=j+1;
            
        end

        % if accepted, save results
        if flag_break==0
            
            SRsuccess=SRsuccess+1;
            Msuccess=Msuccess+1;
            % reorder Qmat according to VAR
            temp=NaN(nvars,nvars);
            temp(:,ix)=Qmat(:,:);

            % output matrix is impact matrix of structural shocks, i.e. effect on 
            % vector y is A0out*shockvec, where shockvec is a unit vector (n x 1).
            % a change in notation is required
            % A0out = inv(A_0)' = inv (h(Sigma)^-1 Q)' = chol(sigma)'*inv(Q')                         
          
            tempA = chol(sigma)\temp; 
            tempE = eye(nvars);
            Anorm = NaN(nvars,nvars);
            
            for jj=1:nvars
               Anorm(:,jj) = tempA(:,jj)/tempA(jj,jj);
            end
               
            for jj=1:nvars
               if tempE(jj,:)/Anorm*tempA(:,jj) < 0
                   Anorm(:,jj) = -Anorm(:,jj);
               end               
            end
                       
            B0drawstore(:,:,SRsuccess)=chol(sigma)'*temp;          
            A0drawstore(:,:,SRsuccess)=Anorm;
                       
            betadrawstore(:,:,SRsuccess)=beta;
            sigmadrawstore(:,:,SRsuccess)=sigma;
        end
    end
    model_acc(1,ii)=Msuccess;
end

A0drawstore=A0drawstore(:,:,1:SRsuccess);
B0drawstore=B0drawstore(:,:,1:SRsuccess);
betadrawstore=betadrawstore(:,:,1:SRsuccess);
sigmadrawstore=sigmadrawstore(:,:,1:SRsuccess);

end

