function [Z,S,ix] = GenRes_ARW_setup(zeroRes,signRes,AzeroRes,AsignRes,nvars)
% SignRes_ARW_setup: Setup of restrictions matrices for SignRes_ARW
%
% Follows Arias, Rubio-Ramirez and Waggoner (2014), Inference based on SVARs 
% identified with sign and zero restrictions, Dynare Working Paper no. 30.
%
% Preparation for calling of function SignRes_ARW
%
% input:
%
% zeroRes [nhorizon, nvars, nvars] matrix of zero restrictions (1
% binding, 0 non-binding). [period restriction is binding, variable for
% which it binds, shock for which it binds].
%
% signRes [nhorizon, nvars, nvars] matrix of sign restrictions (+1
% positive, -1 negative, 0 non-binding). [period restriction is binding, 
% variable for which it binds, shock for which it binds].
%
% nhorizon might be different for zeroRes and for signRes.
% output:
%
% nvars: number of variables in the VAR
%
% output:
% Z,S: matrices of zero and sign restrictions conformable with notation in
% Arias et al. (2014).
%
% flag_rank: checks for identification through zero restrictions using 
% Theorem 7 from Rubio-Ramirez, Waggoner and Zha (2008), Structural Vector 
% Autoregressions: Theory of Identification and Algorithms for Inference,
% Federal Reserve Bank of Atlanta, Working Paper 2008-18,
%
% and
%
% Binning (2013), Underidentified SVAR models: A framework for combining
% short and long-run restrictions with sign-restrictions, Norges Bank,
% Working Paper 2013/4.
%
% 0: overidentified, 1: exactly identified, 2: underidentified   
%
% Note: under- and overidentification can coincide (see the examples in
% Binning, 2013). In this case, the code will indicate only one of the 
% problems - the one that appears first.
%
% Written by Martin Mandler, Deutsche Bundesbank.
%
% Last modified: 05.05.2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% horizon of restrictions
nhz=size(zeroRes,1);
nhs=size(signRes,1);
nhmax=max([nhz,nhs]);

% resort shocks according to number of zero and sign restrictions

ns=NaN(nvars,1);  % ns(j): # sign restrictions for shock j
nz=NaN(nvars,1);  % nz(j): # sign restrictions for shock j

for i=1:nvars
    nz(i)=sum(sum(zeroRes(:,:,i)))+sum(AzeroRes(:,i));
    ns(i)=sum(sum(signRes(:,:,i)~=0))+sum(AsignRes(:,i)~=0);  
end

% first sort according to sign restrictions
[nss, ix] = sort(ns,1,'descend');
nzz=nz(ix); 

% then sort according to zero restrictions (must be first in ordering of
% shocks
[nzzz, ixx] = sort(nzz,1,'descend');
ix=ix(ixx); % ix contains the counting of the shocks ordered according to 
            % nz and ns; important to undo reordering after the algorithm

nzero=sum(nz~=0);   % number of shocks with zero restrictions

% rebuild Z
Z=zeros(nvars*(nhmax+1),nvars*(nhmax+1),nvars);

for ii=1:nvars    
    for i=1:nvars
        for j=1:nhz            
            % build selection vectors for zero restrictions on IRF
            Z(nvars+(j-1)*nvars+i,nvars+(j-1)*nvars+i,ii)=zeroRes(j,i,ii); 
        end 
        % build selection vector for zero restrictions on A0
        Z(i,i,ii) = AzeroRes(i,ii);
    end    
end


% rebuild S
S=zeros(nvars*(nhmax+1),nvars*(nhmax+1),nvars);

for ii=1:nvars
    for i=1:nvars
        for j=1:nhs
            % build selection vectors for sign restrictions on IRF
            S(nvars+(j-1)*nvars+i,nvars+(j-1)*nvars+i,ii)=signRes(j,i,ii); 
        end
        % build selection vector for sign restrictions on A0
        S(i,i,ii) = AsignRes(i,ii);
    end
end


end

