function bvarIrfPlot(irfmat,iFigure,select,addstring,shockname,varnames,legstring)
% bvarIrfPlot: Plots impulse responses from BVAR model
%
% required inputs:
%
% irfmat:       matrix of impulse responses [time x variables x nlines]
%               with the last dimension containing two percentiles of the IRF
%               (at the last two positions) and one or more central series 
%               (mean, median or a selected IRF)
%
% optional inputs:
% iFigure:      counter for Figure number
%
% varselect:    row vector of variables for which to plot IRFs
%
% addstring:    additional string to add to header
%
% shockname:    name of shock (string)
%
% varnames:     names of variables (vector, string)
%
% legstring:    cell of labels for legend (or string if only one label)
%
% last modified: 20.04.2015
%
%##########################################################################


[nhor,nvars,nlines]=size(irfmat);

if isempty(iFigure)
    iFigure=1;
end

if isempty(select)
    select=1:nvars;
end

if isempty(addstring);
    addstring='';
end

if isempty(irfmat)
    disp('No impulse responses available for plotting!');
    return;
end

if size(irfmat,3)<3
    disp('Too few input series for IRF plot.');
    disp('Minimum is three (2 bands and one central series).');
    return;
end;

if isempty(shockname)
    shockname='unnamed shock';
end

TT=0:nhor-1;
zeroline=zeros(nhor);

% Determine how many columns and rows are needed 
Ncols=fix(sqrt(length(select)));
     
if Ncols^2<length(select)
    Nrows=Ncols+1; 
    if Nrows*Ncols<nvars
        Nrows=Nrows+1;
    end
else
    Nrows=Ncols;  
end

figure(iFigure);

for jc=1:length(select)
          
    subplot(Nrows,Ncols,jc);
    co = get(gca,'ColorOrder');
    
    % Change to new colors.
    set(gca, 'ColorOrder', [0 0 0; 0 0 1; 0 1 0; 1 1 0; 1 0 0; 0 1 1; 1 0 1; 1 1 1], 'NextPlot', 'replacechildren');
        
    plot(TT,zeroline,'--k',TT,squeeze(irfmat(:,select(jc),1:end-2))',TT,irfmat(:,select(jc),end-1)','--r',TT,irfmat(:,select(jc),end)','--r','LineWidth', 2);
    axis tight;
    if isempty(varnames)
        titlename=num2str(select(jc));
    else     
        titlename=varnames(select(jc));      
    end
    title(titlename);
       
    if jc==length(select) && ~isempty(legstring)
        legend(legstring,'Location','SouthWest');
    end
    
end


    ha = axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 1], ...
        'Box','off','Visible','off','Units','normalized', 'clipping' , 'off');    

    text(0.5, 1,[strcat('\bf Response to','{ }',shockname,'{ }',addstring)],'HorizontalAlignment', ...
    'center','VerticalAlignment', 'top')
    
    druck = figure(iFigure);
    outputdateiFigRes = strcat('output\IRF_',num2str(iFigure),'.eps');
    print(druck,'-depsc', outputdateiFigRes); 
    
    outputdateiFigRes = strcat('output\IRF_',num2str(iFigure),'.png');
    print(druck,'-dpng', outputdateiFigRes); 
           

