function bvarIrfPlotFill(irfmat,iFigure,select,addstring,shockname,varnames,legstring)
% bvarIrfPlot: Plots impulse responses from BVAR model with shaded
% credible sets
%
% required inputs:
%
% irfmat:       matrix of impulse responses [time,variables,nlines]
%               with the last dimension containing two percentiles of the IRF
%               (at the last two positions) and one or two central series 
%               (mean, median or selected IRF)
%
% optional inputs:
% iFigure:      counter for Figure number
%
% varselect:    row vector of variables for which to plot IRFs
%
% addstring:    additional string to add to header
%
% shockname:    name of shock (string)
%
% varnames:     names of variables (vector, string)
%
% legstring:    cell of labels for legend (or string if only one label)
%
% Last modified: 21.04.2015
%##########################################################################

[nhor,nvars,nlines]=size(irfmat);

if isempty(iFigure)
    iFigure=1;
end

if isempty(select)
    select=1:nvars;
end

if isempty(addstring);
    addstring='';
end

if isempty(irfmat)
    disp('No impulse responses available for plotting!');
    return;
end

if size(irfmat,3)<3
    disp('Too few input series for IRF plot.');
    disp('Minimum is three (2 bands and one central series).');
    return;    
end;

if isempty(shockname)
    shockname='unnamed shock';
end

TT=0:nhor-1;

zeroline=zeros(nhor);
grey = [0.8,0.8,0.8];

% automating styles for lines
styllin=['- ';'--';': ';'-.'];
if size(irfmat,3)>4
  for j=5:size(irfmat,3)   
    styllin=[styllin; styllin(j-4,:)];  
  end
end

stylcol=['b';'k';'g';'r';'y';'m';'c'];
if size(irfmat,3)>7
  for j=8:size(irfmat,3)
    styllin=[stylcol; stylcol(j-7,:)];  
  end
end

% Determine how many columns and rows are needed 
Ncols=fix(sqrt(length(select)));
     
if Ncols^2<length(select)
    Nrows=Ncols+1; 
    if Nrows*Ncols<nvars
        Nrows=Nrows+1;
    end
else
    Nrows=Ncols;  
end

figure(iFigure);

for jc=1:length(select)
          
    subplot(Nrows,Ncols,jc);   
    
    % Change to new colors.    
    hb=fill( [TT fliplr(TT)],  [squeeze(irfmat(:,select(jc),end-1))' fliplr(irfmat(:,select(jc),end)')], grey);
    set(hb,'EdgeColor','None');
    hold on;
    
    plot(TT,zeroline,':k','LineWidth', 2);
    
    for jd=1:size(irfmat,3)-2
        hc(jd)=plot(TT,squeeze(irfmat(:,select(jc),jd))',strcat(stylcol(jd),styllin(jd,1:2)),'LineWidth', 2);
    end
    
    axis tight;
    hd=axis;
    axis([-0.2 hd(2:4)]);   % slight offset to the left to avoid covering the vertical axis with the filled area
    hold off;
    
    if isempty(varnames)
        titlename=num2str(select(jc));
    else     
        titlename=varnames(select(jc));      
    end
    title(titlename);
       
    if jc==length(select) && ~isempty(legstring)
        legend(hc,legstring,'Location','SouthWest');
    end
    
end

    ha = axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 1], ...
        'Box','off','Visible','off','Units','normalized', 'clipping' , 'off');    

    text(0.5, 1,[strcat('\bf Response to','{ }',shockname,'{ }',addstring)],'HorizontalAlignment', ...
    'center','VerticalAlignment', 'top');
    
    druck = figure(iFigure);
    outputdateiFigRes = strcat('output\IRF_',num2str(iFigure),'.eps');
    print(druck,'-depsc', outputdateiFigRes); 
    
    outputdateiFigRes = strcat('output\IRF_',num2str(iFigure),'.png');
    print(druck,'-dpng', outputdateiFigRes); 
           

