function irf = bvarIrfSR(beta,sigma,shockvar,nhorizon,A0mat,const,shocksize)
% bvarIrfSR computes impulse responses using given rotatin matrices, eg
% from sign restrictions
%
% IRFs are computed up horizon nhorizon based on inputs beta and sigma.
% VAR is defined as Y = X*beta + e with Y[Txn], X[Txk], beta[kxn]
%
% if constant included in VAR constant is ordered last, i.e. lags*n+1
% row in beta
%
% inputs:
%
% beta:         matrix of VAR coefficients
% sigma:        covariance matrix of VAR
% shockvar:     position of variable to be shocked (column in Y)
% nhorizon:     maximum horizon of impulse responses; code will return
%               nhorizon+1 which included the impact period (period 0)
% const:        switch [0,1] indicating whether a constant is included; if
%               1 the constant is assumed to be in the last row of beta.
% shocksize:    if defined the shock will be of the size given in
%               shocksize, otherwise it will be a one standard deviation
%               shock
%
% output:
% 
% irf:          [(nhorizon+1) x n] matrix of impulse responses, where n
%               is the number of variables
%
% Written by Martin Mandler, Deutsche Bundesbank.
%
% Last modified: 05.05.2015
%
% #########################################################################

[k,n] = size(beta);

if const==1
 lags = (k-1)/n;
else
 lags = k/n;
end

v=zeros(n,1);  

if exist('shocksize','var') && ~isempty(shocksize)
    if exist('A0mat','var') && ~isempty(A0mat)
        v(shockvar)=shocksize/A0mat(shockvar,shockvar);
    else
        cholsigma=chol(sigma)';
        v(shockvar)=shocksize/cholsigma(shockvar,shockvar);
    end
else 
    v(shockvar)=1;
end

if ~exist('A0mat','var') || isempty(A0mat)
    factor=chol(sigma)';
else 
    factor=A0mat;
end

yhat=zeros(nhorizon+lags+1,n);
in=lags;

for tau=1:nhorizon+1
    xhat=[reshape(yhat([in+tau-1:-1:in+tau-lags],:)',k-1,1)]';
    yhat(in+tau,:)=xhat*beta(1:end-1,:)+(tau==1)*(factor*v)';
end

irf = yhat(in+1:end,:);
