function irf = bvarIrfsVo1(beta,sigma,shockvar,nhorizon,const,shocksize)

% computes impulse responses using cholesky ordering to shock in position 
% nshock up to horizon nhorizon based on inputs beta and sigma.
% VAR is defined as Y = X*beta + e with Y[Txn], X[Txk], beta[kxn]
%
% If constant included in VAR constant is ordered last, i.e. lags*n+1
% row in beta
%
% inputs:
%
% beta:         matrix of VAR coefficients
% sigma:        covariance matrix of VAR
% shockvar:     position of variable to be shocked (column in Y)
% nhorizon:     maximum horizon of impulse responses; code will return
%               nhorizon+1 which included the impact period (period 0)
% const:        switch [0,1] indicating whether a constant is included; if
%               1 the constant is assumed to be in the last row of beta.
% shocksize:    if defined the shock will be of the size given in
%               shocksize, otherwise it will be a one standard deviation
%               shock
%
% output:
% 
% irf:          [(nhorizon+1) x n] matrix of impulse responses, where n
%               is the number of variables
%
% Last modified: 20.04.2015
%
% #########################################################################

[k,n] = size(beta);

if const==1
 lags = (k-1)/n;
 kn=k-1;
else
 lags = k/n;
 kn=k;
end

cholsigma=chol(sigma)';
v=zeros(n,1);  

if exist('shocksize','var') && ~isempty(shocksize)
    v(shockvar)=shocksize/cholsigma(shockvar,shockvar);
else 
    v(shockvar)=1;
end

yhat=zeros(nhorizon+lags+1,n);
in=lags;

for tau=1:nhorizon+1
    xhat=[reshape(yhat([in+tau-1:-1:in+tau-lags],:)',kn,1)]';
    yhat(in+tau,:)=xhat*beta(1:kn,:)+(tau==1)*(cholsigma*v)';
end

irf = yhat(in+1:end,:);
