function [ min_matGs_ind ] = irf_frypagan_mod(S_IRF,select,nhor) 
% irf_frypagan: selection of median model for impulse responses.
%
% Selection of median model for impulse responses based on minimization of 
% sum of squared deviations from median impulse responses.
%
% See Fry, R. and Pagan, A. (2006), "Sign Restrictions in Structural Vector 
% Autoregressions: A Critical Review," Journal of Economic Literature, 
% 938-60, December. 
%
% inputs:
%
% S_IRF:    matrix of impulse responses; dimensions [horizon, variables,
%           shocks, draws]; REQUIRED
%
% select:   matrix of dimension [#variables #shocks] containing the weights
%           assigned to specific variable-shock combinations for the sum of
%           squares minimization; default is a matrix of ones.
%
% nhor:     horizon over which sum of squared deviations criterion is
%           applied; defaults to horizon of impulse responses
%
% output:  
%
% min_matGS_ind:    number of the draw for the VAR coefficients that
%                   minimizes the (weighted) sum of squared deviations from
%                   the median impulse responses.
%
% Last modified: 21.04.2015
%##########################################################################
 
% get horizon, number of variables, number of shocks, number of draws
[ hh, n, nshocks, ndraws ] = size(S_IRF); 

% if no selection is given, all impulse responses enter the selection
if ~exist('select','var') || isempty(select)
    select=ones(n,nshocks);
end
 
% if no horizon is given, impulse responses enter selection over full
% horizon
if ~exist('nhor','var') || isempty(nhor)
   nhor = hh;
elseif nhor>hh
   nhor = hh;
   disp('Length of fitting period reduced to length of impulse responses.');
end

nselect=size(select,2);

mat_irf_draw = NaN(n*nselect,ndraws); 
for ishock = 1:nselect 

    med_S_IRF = zeros(n,nhor); 
    std_S_IRF = zeros(n,nhor); 
    std_irf = NaN(n,nhor,ndraws); 
    mat_draw = NaN(n,nhor);  
    
    for indx = 1:n      
        
        mat_S_IRF = (squeeze(S_IRF(1:nhor,indx,ishock,:)))'; 
        med_S_IRF(indx,:) = median(mat_S_IRF);         
        std_S_IRF(indx,:) = std(mat_S_IRF); 
                
        % if all IRFs are equal, e.g. because of identification
        % assumptions, std_S_IRF will be zero for these elements
        % and the selection criterion will be undefined. Therefore,
        % we replace std_S_IRF for these entries with a positive number.
        
        std_S_IRF(indx,:)=std_S_IRF(indx,:)+1*(std_S_IRF(indx,:)==0);
        
        for idraw = 1:ndraws 
            std_irf(indx,:,idraw) = (squeeze(S_IRF(1:nhor,indx,ishock,idraw))'-med_S_IRF(indx,:)) ... 
                                         ./ std_S_IRF(indx,:); 

            mat_draw(indx,idraw) = select(indx,ishock)* std_irf(indx,:,idraw) * (std_irf(indx,:,idraw))';
            
        end 
    end
        
    mat_irf_draw(n*(ishock-1)+1:n*ishock,:) = mat_draw; 

end
   
matGs = sum(mat_irf_draw); 
[ min_matGs, min_matGs_ind ] = min(matGs);  



