function [logML,betahat,sigmahat,epshat,d,omega]=logMLVAR_forminVo1(par,defaultpar,y,x,lags,T,n,b,MIN,MAX,SS,Vc,pos,mn,sur,suron,noc,nocon,y0,hyperpriors,priorcoef)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the log-posterior (or the logML if hyperpriors=0), 
% the posterior mode of the coefficients and the covariance matrix of the 
% residuals of the BVAR of Giannone, Lenza and Primiceri (2015), Prior 
% Selection for Vector Autoregressions, Review of Economics and Statistics. 
%
%
% Modified by Martin Mandler and Michael Scharnagl, Deutsche Bundesbank.
% 
% Original code by Giannone, Lenza and Primiceri.
%
% Modified to order constant last among coefficients, to allow for a fixed
% value of lambda and to allow for easier handling of fixing
% hyperparameters at default values.
%
% Last modified: 20/04/2014
%
% inputs:
%
% par:          vector of hyperparameters
% defaultpar:   default values for hyperparameters
% y:            Txn matrix of LHS variables in VAR  (y = X*beta + e)
% x:            Txk matrix of explanatory variables (k = n*lags +1)
% lags:         lag order
%
% T:            number of observations (after adjusting for lags)
% n:            number of variables
% b:            vector of prior means of VAR coefficients
% MIN:          vector of lower bounds of hyperparameters
% MAX:          vector of upper bounds of hyperparameters
% SS:           vector of AR(1) residual variances 
% Vc:           prior variance on constants
% pos:          position of variables in first differences; prior mean on
%               coefficient on first lag will be set to zero instead of one 
%               under Minnesota prior          
% mn:           structure of switches for hyperparameters (estimated/fixed)
% sur:          switch for initial-dummy-observations prior
% noc:          switch for sum-of-coefficients prior
% y0:           mean of first (lags) values of y
% hyperpriors:  switch for inclusion of hyperpriors (function computes
%               posterior instead of likelihood)
% priorcoef:    structure of parameters of hyperpriors
%
% outputs:
%
% logML:        log marginal likelihood/log posterior
% betahat:      posterior mode of VAR coefficients
% sigmahat:     posterior mode of the VAR covariance matrix
% epshat:       posterior mode of VAR Residuals
% d:            degrees of freedom adjustment
% omega:        vector --> conditional on Sigma beta is distributed as 
%               N(betahat,kron(Sigma,diag(Omega))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% hyperparameters
%

d=n+2;              

% if hyperparameter noch included in optimization set to default values
if mn.lambda==1
    lambda=MIN.lambda+(MAX.lambda-MIN.lambda)/(1+exp(-par(1)));
    jshift=1;
else
    lambda=defaultpar.lambda;
    jshift=0;
end

if mn.psi==0;
    psi=SS*(d-n-1);
    if sur==1;
        theta=MIN.theta+(MAX.theta-MIN.theta)/(1+exp(-par(jshift+1)));        
        if noc==1;
            miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-par(jshift+2)));            
        elseif noc==0 && nocon==1
            miu=defaultpar.miu;
        end
    elseif sur==0;
        if suron==1
            theta=defaultpar.theta;
        end
        if noc==1;
            miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-par(jshift+1)));
        elseif noc==0 && nocon==1
            miu=defaultpar.miu;
        end
    end
elseif mn.psi==1;
    psi=MIN.psi+(MAX.psi-MIN.psi)./(1+exp(-par(jshift+1:n+jshift)));
    if sur==1;
        theta=MIN.theta+(MAX.theta-MIN.theta)/(1+exp(-par(n+jshift+1)));
        if noc==1;
            miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-par(n+jshift+2)));
        elseif noc==0 && nocon==1
            miu=defaultpar.miu;
        end
    elseif sur==0;
        if suron==1
            theta=defaultpar.theta;
        end
        if noc==1;
            miu=MIN.miu+(MAX.miu-MIN.miu)/(1+exp(-par(n+jshift+1)));
        elseif noc==0 && nocon==1
            miu=defaultpar.miu;
        end
    end
end

if mn.alpha==0;
    alpha=defaultpar.alpha;
elseif mn.alpha==1;
    alpha=MIN.alpha+(MAX.alpha-MIN.alpha)/(1+exp(-par(end)));
end

%% setting up the priors
%

k=1+n*lags;
omega=zeros(k,1);
omega(k)=Vc;
for i=1:lags
    omega((i-1)*n+1:i*n)=(d-n-1)*(lambda^2)*(1/(i^alpha))./psi;
end

% prior scale matrix for the covariance of the shocks
PSI=diag(psi);

Td=0;
xdsur=[];
ydsur=[];
xdnoc=[];
ydnoc=[];

% dummy observations if sur and/or noc = 1
if suron==1 || sur==1;
    
    xdsur=[(1/theta)*repmat(y0,1,lags) 1/theta];
    ydsur=(1/theta)*y0;
   
    y=[y;ydsur];
    x=[x;xdsur];
    Td=1;
end

if nocon==1 || noc==1;
        
    ydnoc=(1/miu)*diag(y0); ydnoc(pos,pos)=0;
    xdnoc=[(1/miu)*repmat(diag(y0),1,lags) zeros(n,1)];
    
    y=[y;ydnoc];
    x=[x;xdnoc];
    Td=Td+n;
end

T=T+Td;

%% output
%

% posterior mode of the VAR coefficients (below eq. A.7 in the appendix)
betahat=(x'*x+diag(1./omega))\(x'*y+diag(1./omega)*b);

% VAR residuals at posterior mode
epshat=y-x*betahat;

% Posterior mode of the covariance matrix (eq. A.8 in the appendix)
sigmahat=(epshat'*epshat + PSI + (betahat-b)'*diag(1./omega)*(betahat-b));

% logML (terms in eq. A.14 in the appendix)
aaa=diag(sqrt(omega))*(x'*x)*diag(sqrt(omega));
bbb=diag(1./sqrt(psi))*(epshat'*epshat + (betahat-b)'*diag(1./omega)*(betahat-b))*diag(1./sqrt(psi));

eigaaa=real(eig(aaa)); eigaaa(eigaaa<1e-12)=0; eigaaa=eigaaa+1;
eigbbb=real(eig(bbb)); eigbbb(eigbbb<1e-12)=0; eigbbb=eigbbb+1;


% eq. A.14 in the appendix; note that T is already adjusted for lag length, 
% ie T is equal to T-p in the paper
logML = - n*T*log(pi)/2 + sum(gammaln((T+d-[0:n-1])/2)-gammaln((d-[0:n-1])/2)) +...
    - T*sum(log(psi))/2 - n*sum(log(eigaaa))/2 - (T+d)*sum(log(eigbbb))/2;


% adjust the normalizing constant if the sur and noc priors are included
if sur==1 || noc==1;
    yd=[ydsur;ydnoc];
    xd=[xdsur;xdnoc];
    
    % prior mode of the VAR coefficients
    % betahatd=(xd'*xd+diag(1./omega))\(xd'*yd+diag(1./omega)*b);
    betahatd=b;     % this is the case for our priors (the line above delivers the same but is numerically not very stable)
    
    % VAR residuals at the prior mode
    epshatd=yd-xd*betahatd;
    
    aaa=diag(sqrt(omega))*(xd'*xd)*diag(sqrt(omega));
    bbb=diag(1./sqrt(psi))*(epshatd'*epshatd + (betahatd-b)'*diag(1./omega)*(betahatd-b))*diag(1./sqrt(psi));
    
    eigaaa=real(eig(aaa)); eigaaa(eigaaa<1e-12)=0; eigaaa=eigaaa+1;
    eigbbb=real(eig(bbb)); eigbbb(eigbbb<1e-12)=0; eigbbb=eigbbb+1;
    
    % normalizing constant
    norm = - n*Td*log(pi)/2 + sum(gammaln((Td+d-[0:n-1])/2)-gammaln((d-[0:n-1])/2)) +...
           - Td*sum(log(psi))/2 - n*sum(log(eigaaa))/2 - (Td+d)*sum(log(eigbbb))/2;
    
    logML=logML-norm;
end


% adjust normalizing constant if hyperpriors are used
if hyperpriors==1;
    logML=logML+logGammapdf(lambda,priorcoef.lambda.k,priorcoef.lambda.theta);
    if sur==1;
        logML=logML+logGammapdf(theta,priorcoef.theta.k,priorcoef.theta.theta);
    end
    if noc==1;
        logML=logML+logGammapdf(miu,priorcoef.miu.k,priorcoef.miu.theta);
    end
    if mn.psi==1;
        logML=logML+sum(logIG2pdf(psi/(d-n-1),priorcoef.alpha.PSI,priorcoef.beta.PSI));
    end
end
logML=-logML;


function r=logGammapdf(x,k,theta)
r=(k-1)*log(x)-x/theta-k*log(theta)-gammaln(k);

function r=logIG2pdf(x,alpha,beta)
r=alpha*log(beta)-(alpha+1)*log(x)-beta./x-gammaln(alpha);